/***************************************************************

	Windows version of Fishnet( Ported from Linux Version )
	Author: Rajesh Hegde K ( krhegde@cs.washington.edu )
	April 21, 2002

***************************************************************/

#pragma once

/* fishguts.h - Fishnet internals*/


/* headers to handle structures listed here */
#include <stdio.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <sys/timeb.h>
#define bzero(a,len) memset(a,0,len)
#include <io.h>
#define STDIN_FILENO 0
#include <sys/types.h>
#include "WinFish.h"

#define DEFAULT_PORT 7777
#define DEFAULT_HOST 0
#define MAX_NEIGHBORS 12

struct neighbor 
{
  int address;                    /* chosen address, HOST order */
  struct in_addr node_ip_address; /* ip address NETWORK order */
  unsigned short node_udp_port;   /* udp address NETWORK order */
  unsigned short lossrate_x_1000; /* loss rate across this link */
};

struct joinrequest 
{
  int node_address;               /* chosen address, HOST order */
  unsigned short node_udp_port;   /* udp address NETWORK order */
  short reserved;                 /* 32-bit alignment padding */
  char username[8];               /* for ownership reporting */
  char domain[8];                 /* for hierarchical topologies */
};

#define FISH_MSG_TYPE_ERROR        0
#define FISH_MSG_TYPE_NEIGHBORLIST 1
#define FISH_MSG_TYPE_JOINREQUEST  2
#define FISH_MSG_TYPE_MAX          3 /* upper bound */

struct fish_message 
{
  unsigned short length;    /* HOST order (may change later) */
  unsigned short type;
  union 
  {
    /* zero size arrays terminating a message can be
       referenced with a longer array index */
    char errormessage[0]; /* null terminated, 0 included in
                             length */
    neighbor neighbors[0]; /* terminated by message
                                     length */
    joinrequest joinreq; 
  } body;
  /* put nothing here  (neighbors will be out-of-bounds) */
};

#define MIN_MESSAGE_SIZE 4
#define DEBUG_PRINTF(x) do {} while(0)


/* in Winfishtimer.c */
void fish_executepending(void);
int fish_time_to_next_event( struct timeval *delta_t );

/* in Winfishdebug.c */
extern void printmessage(const struct fish_message *pfm);
extern void printneighbors(const struct neighbor *ngh);

extern void SysTimeToTimeVal( SYSTEMTIME* psystime, timeval* ptimeval );

void errorabort(const char* msg);
static DWORD __stdcall kbrdthread( void );

