/**
 * Example Fishnet program
 * CSE588
 * Spring 2002
 */
#include <stdio.h>
#include "winfish.h"
#include <string.h>
#include <stdlib.h>
#include <winsock2.h>




#include "echo-protocol.h"



/*
 * Handle a packet that is meant for me.
 */
void handle_packet(int from, struct packet* p_pkt, int length) 
{
  switch(p_pkt->hdr.protocol) 
  {
    case FISH_PROTOCOL_ECHO_REQUEST:
		
		handle_echo_request(p_pkt);
      
		break;
  
	case FISH_PROTOCOL_ECHO_RESPONSE:
	
		handle_echo_response(p_pkt);
	   
		break;

	default:
      
		fish_debug(FISH_DEBUG_NONE, "Unexpected protocol: %i\n",p_pkt->hdr.protocol);

      break;
  }
}

/* 
 * Handle keyboard input; called by fish_main 
 */
void on_input(char* text) 
{
  if (echo_input(text)) 
  {
    // This was a send command.
  } 
  else 
  {
    // Unknown command; print error message
    printf("Commands:\n  ""send <dest> <msg>\n");
  }
}

/* 
 * Receive a frame and handle or forward it; called by fish_main 
 */
void on_receive(int from, void* frame, int length) 
{
  struct packet *p_pkt = (struct packet *)frame;

  // Checking the protocol field helps to protect us against naked frames
  // or other non-packet data.
  if (p_pkt->hdr.protocol < FISH_PROTOCOL_ERROR || p_pkt->hdr.protocol >= FISH_PROTOCOL_MAX) 
  {
    fish_debug(FISH_DEBUG_NONE,  "received frame with invalid protocol; discarding\n");
    return;
  }

  if (p_pkt->hdr.dst == fish_getaddress() || p_pkt->hdr.dst == ALL_NEIGHBORS) 
  {
    handle_packet(from, (packet*)frame, length);
  } 
  else 
  {
    p_pkt->hdr.ttl--;
    if (p_pkt->hdr.ttl > 0) 
	{
      forward(p_pkt, length);
    }
  }
}

void print_usage(void) 
{
  printf("Usage: ./hw1-sample headhost:headport domain address\n");
}

int main(int argc, char** argv) 
{
  if (argc < 4) 
  {
    print_usage();
    exit(-1);
  }

  WORD wVersionRequested;
  WSADATA wsaData;
  int err;

  wVersionRequested = MAKEWORD( 2, 2 );

  err = WSAStartup( wVersionRequested, &wsaData );
  if ( err != 0 ) 
  {
	/* Tell the user that we could not find a usable */
	/* WinSock DLL.                                  */
	return -1;
  }

  fish_keybhook(on_input);
  fish_recvhook(on_receive);

  fish_joinnetwork(argv[1], argv[2], atoi(argv[3]));

  fish_main();

  WSACleanup();

  return 0;
}
